// -*-c++-*-

#include "MorphVec3.h"


/*
 * Set morph coordinates.
 */
void MorphVec3::setCoords(int ncoord, osg::Vec3* source, osg::Vec3* target)
{
  freeCoord();
  _ncoord = ncoord;

  // Source normals
  _coord[0] = new osg::Vec3[ncoord];
  memcpy(_coord[0], source, ncoord * sizeof(osg::Vec3));

  // Target normals
  _coord[1] = new osg::Vec3[ncoord];
  memcpy(_coord[1], target, ncoord * sizeof(osg::Vec3));

  // Preset with source normals
  _coord[2] = new osg::Vec3[ncoord];
  update(0.0f);
}


/*
 * Update morph buffers
 */
void MorphVec3::update(float t)
{
#define TIMING 0
#if TIMING == 1
  static osg::Timer timer;
  osg::Timer_t start = timer.tick();
#endif

  // Morph coordinates and normals
  if (t <= 0.0f) {
    // Shortcut: Use source
    memcpy(_coord[2], _coord[0], _ncoord * sizeof(osg::Vec3));
  }
  else if (t >= 1.0f) {
    // Shortcut: Use target
    memcpy(_coord[2], _coord[1], _ncoord * sizeof(osg::Vec3));
  }
  else {
    // Interpolate
    for (int i = 0; i < _ncoord; i++) {
      _coord[2][i] = _coord[0][i] + (_coord[1][i] - _coord[0][i]) * t;
    }
  }

  // Update geometries
  for (unsigned int i = 0; i < _geodes.size(); i++)
    if (_geodes[i]) {
      osg::Geode* geode = _geodes[i];

      // Force recompile of Drawable display list
      for (int j = 0; j < geode->getNumDrawables(); j++) {
	geode->getDrawable(j)->dirtyDisplayList();
      }

      // Force recompute of bounding box
      geode->dirtyBound();
    }

#if TIMING == 1
  static osg::Timer_t duration = 0;
  static float count = 0.0f;
  duration += timer.tick() - start;
  count += 1.0f;
  cerr << (float) duration / count << endl;
#endif
}


/*
 * Free coordinate buffers
 */
void MorphVec3::freeCoord()
{
  for (int i = 0; i < 3; i++) {
    if (_coord[i])
      delete[] _coord[i];
    _coord[i] = NULL;
  }
  _ncoord = 0;
}
