// -*-c++-*-

#ifndef _MORPHVEC3_H_
#define _MORPHVEC3_H_

#include "osg/Vec3"
#include "osg/Geode"

#include <vector>


/**********************************************************************
 *
 * Morphing Vec3
 *
 **********************************************************************/
class MorphVec3
{
public:
  /// Constructor.
  MorphVec3() {
    _ncoord = 0;
    _coord[0] = _coord[1] = _coord[2] = NULL;
  }

  /// Destructor.
  ~MorphVec3() {
    freeCoord();
  }

  /**
   * Set morph coordinates.
   * Initializes morph buffer to source coordinates.
   * @param ncoord Number of coordinates in source and target.
   * @param source Source coordinates.
   * @param target Target coordinates.
   */
  void setCoords(int ncoord, osg::Vec3* source, osg::Vec3* target);

  /// Returns number of morphed coordinates.
  const int getNumCoords() const { return _ncoord; }

  /// Returns pointer to morphed coordinate buffer.
  const osg::Vec3* getMorphCoords() const { return _coord[2]; }

  /**
   * Add Geode to list of objects this morphed geometry is used.
   */
  void addGeode(osg::Geode* geode) { _geodes.push_back(geode); }

  /**
   * Update morph buffers.
   */
  void update(float t);

private:
  int _ncoord;
  osg::Vec3* _coord[3];

  typedef std::vector<osg::Geode*> GeodeList;
  GeodeList _geodes;

  /// Free coordinate buffers.
  void freeCoord();
};

#endif
